print("import: 'pyramid_jinja2'")
import pyramid_jinja2

print("import: 'pyramid_jinja2.demo'")
import pyramid_jinja2.demo

print("import: 'pyramid_jinja2.scaffolds'")
import pyramid_jinja2.scaffolds

print("import: 'pyramid_jinja2.tests'")
import pyramid_jinja2.tests

